use http::{header::CONTENT_TYPE, response::Builder, Response};
use http_body_util::Full;
use hyper::body::Bytes;
use serde::Serialize;
use std::convert::Infallible;

pub struct Responder;

impl Responder {
    pub fn not_found() -> Result<Response<Full<Bytes>>, Infallible> {
        Self::response_using_builder(Self::create_builder(404), "Not Found")
    }

    pub fn unathorized() -> Result<Response<Full<Bytes>>, Infallible> {
        Self::response_using_builder(Self::create_builder(401), "Unathorized")
    }

    pub fn text(response: &str) -> Result<Response<Full<Bytes>>, Infallible> {
        Self::response_using_builder(Self::create_builder(200), response)
    }

    pub fn json<T>(json: &T) -> Result<Response<Full<Bytes>>, Infallible>
    where
        T: Serialize,
    {
        Self::json_using_status(200, json)
    }

    pub fn text_using_status(
        status: u16,
        response: &str,
    ) -> Result<Response<Full<Bytes>>, Infallible> {
        Self::response_using_builder(Self::create_builder(status), response)
    }

    pub fn json_using_status<T>(status: u16, json: &T) -> Result<Response<Full<Bytes>>, Infallible>
    where
        T: Serialize,
    {
        let builder = Self::create_builder(status).header(CONTENT_TYPE, "application/json");

        Self::response_using_builder(builder, &serde_json::to_string(json).unwrap())
    }

    fn response_using_builder(
        builder: Builder,
        response: &str,
    ) -> Result<Response<Full<Bytes>>, Infallible> {
        Ok(builder
            .body(Full::<Bytes>::from(response.to_string()))
            .unwrap())
    }

    fn create_builder(status: u16) -> Builder {
        Response::builder().status(status)
    }
}
